<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: create_cvs.php 4283 2009-12-31 23:08:59Z mpapayannis $
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

/**
 * @author Daniel Perez <danielperez.arg@gmail.com>
 *
 * @version   $Id: create_cvs.php 4283 2009-12-31 23:08:59Z mpapayannis $
 */

require_once('lib/initialize.php');
require_once('system/Cronjob.php');
require_once('core/ProjectPeer.php');
require_once('core/PluginPeer.php');
require_once('core/ProjectPluginPeer.php');
require_once('plugins/scm/lib/ProjectScmPeer.php');
require_once('plugins/scmcvs/lib/CVSRepository.php');
require_once('GFConf.php');
require_once('plugins/scmcvs/lib/exceptions/CVSException.php');

class CreateCVSRepositoriesCronjob extends Cronjob {
	private $cvsdir_prefix,$plugins_path,$cvs_binary_version,$repositoryPath;
	const PLUGIN_NAME = "scmcvs";
	
	public function __construct() {
		$this->jobname = basename(__FILE__);
		parent::__construct();
	}
	
	public function run() {
		$this->cleanup();

		$this->cvsdir_prefix = GFConf::get('scmcvs.localCVSRoot');
		$this->plugins_path = GFConf::get('system.pluginsPath');
		$this->cvs_binary_version = GFConf::get('scmcvs.binaryVersion');
		// Loop through and create/update each repository for every project that uses SCMCVS plugin
		
		if (is_dir($this->cvsdir_prefix)) {
			chmod($this->cvsdir_prefix, 0755);		// this allows repository browsing
			$this->updateCvsRepositories();
		} else {
			if (is_file($this->cvsdir_prefix)) {
				$this->logCron($this->cvsdir_prefix." exists but is a file");
				$this->commitLog();
				exit();
			} else {
				if (mkdir($this->cvsdir_prefix)) {
					//need to update group permissions using chmod
					$this->updateCvsRepositories();
				} else {
					$this->logCron("Unable to create ".$this->cvsdir_prefix." directory");
					$this->commitLog();
					exit();
				}
			}
		}
	}
	

	private function updateCvsRepositories() {
		// lets get all projects that use the plugin
		$c = new Criteria();
		$c->add(PluginPeer::PLUGIN_NAME, self::PLUGIN_NAME);
		$project_plugins = ProjectPluginPeer::doSelectJoinPlugin($c);
		
		foreach ($project_plugins as $project_plugin) {
			$project = $project_plugin->getProject(); // get the project
			$cvsplugin = $project->getPluginByName('scmcvs');
			
			$repos = new CVSRepository($project);
			// CVS not configured for this project
			if (is_null($cvsplugin) || !$repos->isConfigured()) {
				continue;
			}

			$project_scm = $repos->getScmConfiguration(); // get the scm configuration for this project
			
			// If we're splitting SCM servers, see if the repository for this project must be
			// created on this server
			if (GFConf::get('system.splitScmServers') && GFConf::varExists('scmcvs.scmServerId')) {
				$repository_server_id = $project_scm->getScmServer()->getScmServerId();
				// This repository belongs to another server, don't create it
				if ($repository_server_id != GFConf::get('scmcvs.scmServerId')) {
					continue;
				}
			}

			try {
				if ($project->isActive() && $project_scm->getControlled()) {
					if (!$repos->isCreated()) {
						$permissions = ($project->getIsPublic()) ? 02775 : 02770;
						$repos->create($permissions);
					} else {
						$repos->updateRepository();
					}

				}
			} catch (CVSException $e) {
				$this->logCron($e->getMessage());
			}
		}
		
		$this->commitLog();
	}
	
	
	public function cleanup() {
		$c = new Criteria();	
		$c->add(DeletedProjectPeer::ISDELETED, 0);	
		$deletedProjects = DeletedProjectPeer::doSelect($c);
		$cvsroot = GFConf::get('scmcvs.localCVSRoot');

		$deletedRoot = SystemUtils::buildFilePath($cvsroot, '/.deleted');
		if (!is_dir($deletedRoot)) {
			mkdir($deletedRoot);
		}

		$output = array();
		
		foreach ($deletedProjects as $deletedProject){
			$deletedUnixName = $deletedProject->getUnixGroupName();

			$reposPath = SystemUtils::buildFilePath($cvsroot, $deletedUnixName);
			if (!is_dir($reposPath)) {
				// Repository doesn't exist
				continue;
			}

			$deletedPath = SystemUtils::buildFilePath($deletedRoot, '/'.$deletedUnixName);

			SystemUtils::execute('mv -f '.$reposPath.' '.$deletedPath, $output);
			$deletedProject->delete();
		}	

		SystemUtils::execute('chown -R root:root '.$deletedRoot, $output);
		SystemUtils::execute('chmod -R o-rwx '.$deletedRoot, $output);
	}
}

$cronjob = new CreateCVSRepositoriesCronjob();
$cronjob->run();
?>
